/*
  ZUNO_HDC1080.cpp v.1.01 - Arduino library support for Temp./Humidity sensor TI HDC1080
  - basic version -
  Copyright (C)2018 SmartHome.Work, Michael Pruefer. All right reserved
  
  This library has been made to make it easy to use the HDC1080 chip.
  Chip maybe integrated at CCS811 module for temperature/humidity compensation of
  CCS811  chip, then library could be included here.
  
  Based on the TI datasheet http://www.ti.com/lit/ds/symlink/hdc1080.pdf.
  Tested with Z-Uno and CCS811 module CJMCU-8128

  Permission is hereby granted, free of charge, to any person obtaining a copy of
  this software and associated documentation files (the "Software"), to deal in
  the Software without restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the 
  Software, and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be included 
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION 
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
*/


#include <ZUNO_HDC1080.h>
#include <Wire.h>

#define DBG_HDC 0

byte  HDC1080_tdata[4];

HDC1080::HDC1080(uint8_t addr)
{
  _addr = addr;
  
}

void HDC1080::begin(void)
{
	Wire.begin();
}

void HDC1080::defaultInit(void)
{
	Wire.beginTransmission(_addr);
	Wire.write(HDC1080_REG_CONFIG);
	Wire.write(0x90);  				// set register to software reset and 14 bit resolution
	Wire.write(0x00);
	Wire.endTransmission();
	delay(15);										//device startup time
}

void HDC1080::readAllValues(void)
{
	hdc_readn(HDC1080_REG_TEMP, 4);  // read all values - first 2 bytes temp., second two bytes hum.
									 // use this for more effective reading
}	


float HDC1080::readTemp(void)
{
	hdc_readn(HDC1080_REG_TEMP, 2);
	uint16_t temp;
	temp = (unsigned int)HDC1080_tdata[0] << 8 | HDC1080_tdata[1];  	//high and low byte
	
	return (float)(temp) * 165 / 65536 - 40;							//see datasheet
}

float HDC1080::readHum(void)
{
	hdc_readn(HDC1080_REG_HUM, 2);
	uint16_t hum;
	hum = (unsigned int)HDC1080_tdata[0] << 8 | HDC1080_tdata[1];		//high and low byte
	
	return (float)(hum)/( 65536 )* 100;									//see datasheet
	
}

//Bit 11 of the CONFIG register define the temperature  resolution
void HDC1080::setTempRes(int resolution)
{ 
	uint8_t currConfig;
	hdc_readn(HDC1080_REG_CONFIG,1);
	currConfig = HDC1080_tdata[0];     // bit [15:8]
	HDC1080_tdata[1]=0x00;
	
	switch(resolution)
	{
		case 14:
			currConfig = (currConfig & 0xFB);
			break;
			
		case 11:
			currConfig = (currConfig & 0xFB);
			currConfig = (currConfig | 0x04);  
			break;
	
		default:
			currConfig = (currConfig & 0xFB);		//default: 14bit
	}
	currConfig = (currConfig & 0x7F);
	currConfig = (currConfig | HDC1080_SOFT_RESET);  // softwareReset and new parameters
	
	HDC1080_tdata[0]=currConfig;
	hdc_writeReg(HDC1080_REG_CONFIG, 2);

	delay(15);										//device startup time
}
//  Bits 9 and 10 of the CONFIG register define the humidity resolution
void HDC1080::setHumidRes(int resolution)
{ 
	uint8_t currConfig;
	hdc_readn(HDC1080_REG_CONFIG,1);
	currConfig = HDC1080_tdata[0];     			// bit [15:8]
	HDC1080_tdata[1]=0x00;						// must be 0x00 (datasheet)
	
	switch(resolution)
	{
		case 14:
			currConfig = (currConfig & 0xFC);
			break;
			
		case 11:
			currConfig = (currConfig & 0xFE);
			currConfig = (currConfig | 0x01);  
			break;
			
		case 8:
			currConfig = (currConfig & 0xFD);
			currConfig = (currConfig | 0x02); 
			break;
			
		default:
			currConfig = (currConfig & 0xFC);		//default: 14bit
	}
	currConfig = (currConfig & 0x7F);
	currConfig = (currConfig | HDC1080_SOFT_RESET);  // softwareReset and new parameters

	HDC1080_tdata[0]=currConfig;
	hdc_writeReg(HDC1080_REG_CONFIG, 2);

	delay(15);										//device startup time
}




void HDC1080::hdc_readn(byte reg, byte n) 
{
	Wire.beginTransmission(_addr);
	Wire.write(reg);
	Wire.endTransmission();  					// send register value
	delay(250);									// give time to convert values (for 14bit!!)
	Wire.requestFrom(_addr, n);					// send data n-bytes read
	delay(50);
	for(byte i=0;i<n;i++){
      delayMicroseconds(30);
      HDC1080_tdata[i] = Wire.read(); 			// receive DATA
	}
	Wire.endTransmission();	
	#if DBG_HDC
	Serial.print("<< ");
	Serial.dumpPrint(HDC1080_tdata,n);
	Serial.println("");
	#endif
}


void HDC1080::hdc_writeReg(byte reg, byte n)
{
  Wire.beginTransmission(_addr);	
  Wire.write(reg);						// send register value
  for(byte i=0;i<n;i++){
      delayMicroseconds(30);
      Wire.write(HDC1080_tdata[i]); 	// write DATA
  }
  Wire.endTransmission();				
}